<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Teknisa\SM\Util\ConsultaInscricaoRF;
use Teknisa\Libs\Util\NewCode;
use Teknisa\SM\Service\Sm00005000ParametrizacoesServicesService;

class ger45000ProspectController extends SMCrud {
    
    protected $dataSourceName = 'prospect_hierarquiaPerfilDeAcesso';
    protected $dataSourceNameToSave = 'prospect';
    protected $newCode;
    protected $environment;
    protected $entityManager;
    protected $connection;
    protected $prospectService;
    protected $parametrogeral;
    
    public function __construct ($crudService, $environment, $entityManager, $newCode, $prospectService, Sm00005000ParametrizacoesServicesService $parametrogeral){
        parent::__construct($crudService, $environment);
        $this->newCode          = $newCode;
        $this->entityManager    = $entityManager;
        $this->environment      = $environment;
        $this->connection       = $entityManager->getConnection();
        $this->prospectService  = $prospectService;
        $this->parametrogeral   = $parametrogeral;
    }
    
    public function find(Request\Filter $request, Response $response){
        try{
            $idUtilizaOper = $this->parametrogeral->getIdUtilizaHierarquiaOper($this->environment->getNrorg());
            
            if($idUtilizaOper == 'N'){
                $this->dataSourceName = 'prospect';
            } else { 
                $nrparcnegocio = $this->prospectService->getNrParcNegocio($this->environment->getCdOperador(), $this->environment->getNrorg());
                $request->getFilterCriteria()->addCondition("NRPARCNEGOCIO", "=", $nrparcnegocio);
            }
            
            parent::find($request, $response); 
            $this->dataSourceName = 'prospect_hierarquiaPerfilDeAcesso';
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        
        $this->connection->beginTransaction();
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
            
                if($row['__is_new']){
                    $row['CDPROSPECT'] = $this->newCode->getCodeByTableName("PROSPECT", $row["NRORG"]);
                    $row['NRPARCNEGOCIO'] = $this->newCode->getCodeByTableName("PARCNEGOCIO", $row["NRORG"]);
                    $row['NRSEQCONTATO'] = $this->newCode->getCodeByTableName("NRSEQCONTATO", $row["NRORG"]);
                    $row['NRPARCERIA'] = $this->newCode->getCodeByTableName("PARCERIA", $row["NRORG"]);
                    $row['CDOPERINCLUSAO'] = isset($row['CDOPEREVOL']) ? $row['CDOPEREVOL'] : $this->environment->getCdOperador();
                    $row['NRCONTATO'] = $row['NRSEQCONTATO'];
                    $row["CDTIPOPARCEIRO"] = "PROSPECT";
                    $row['ISCPF'] = ($row['IDTPIJURPROS'] === 'F') ? 'S' : 'N';
                    $row['TIPOINSC'] = ($row['IDTPIJURPROS'] === 'F') ? 'CPF' : ($row['IDTPIJURPROS'] === 'J') ? 'CNPJ' : 'LIVRE';
                    $row['NMFANTPROS'] = empty($row['NMFANTPROS']) ? $row['NMRAZSOCPROS'] : $row['NMFANTPROS'];
                    $row['IDLIGEXTPROS'] = 'S';
                    $row['IDPRIPUBPROS'] = 'V';
                    $nrorg = $row["NRORG"];
                    $operador =  $row['CDOPERINCLUSAO'];
                    $nrparcnegocio = $row['NRPARCNEGOCIO'];
                    
                    /* Interlocutor */
                    $nomeInterlocutor = $row['NMINTEPROS'];
                    $emailinterloc = $row['DSEMAILIPROS'];
                    $telInterloc = $row['NRTEL1INPROS'];
                    
                    /* Endereço */
                    $cep = $row['NRCEPENDERECO'];
                    $endereço = $row['DSENDERECO'];
                    $numero = $row['NRIMOVELENDERECO'];
                    $complemento = $row['DSCOMPLEENDERECO'];
                    $pais = $row['CDPAIS'];
                    $estado = $row['SGESTADO'];
                    $municipio = $row['CDMUNICIPIO'];
                    $bairro = $row['CDBAIRRO'];
                    $nmbairro = $row['NMBAIRRO'];
                    
                    $this->prospectService->setParceiroNegocio($row);
                    $this->prospectService->newContato($row);
                    $this->prospectService->createPartnership($row);
                    
                    if ($nomeInterlocutor != null){
                        $nrseqinterlocutor = $this->newCode->getCodeByTableName("CRM_INTERLOCUTOR", $nrorg);
                        $this->prospectService->inserecrmInterlocutor($nrseqinterlocutor, $nrparcnegocio, $nomeInterlocutor, $emailinterloc, $telInterloc, $nrorg, $operador);
                    };
                    if ($cep != null){
                        $nrenderecoparc = $this->newCode->getCodeByTableName("ENDERECOPARC", $nrorg);
                        $this->prospectService->insereEnderecoParc($nrenderecoparc, $nrparcnegocio, $cep, $endereço, $numero, $complemento, $pais, $estado, $municipio, $bairro, $nmbairro, $nrorg, $operador);
                    };
                }
            }
            
            parent::save($request, $response);
            
            $this->connection->commit();
            
            $response->addDataSet(new DataSet("DADOS", array("PROSPECT" => $row['CDPROSPECT'])));
        } catch(\Exception $e) {
            $this->connection->rollback();
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    public function selectContatoProspect(Row $row, Response $response)
    {
        $row = $row->getRow();
        $retorno = $this->prospectService->prepareSelectProspectContact($row);
        $response->addDataSet(new DataSet("DADOS", $retorno));
    }
    
    public function insertNrseqcontatoForNewOpportunity(Row $row){
        
        $row = $row->getRow();
        $this->prospectService->callForInsertNrseqcontatoForNewOpportunity($row);
        
    }
    
    public function updatePartnerType(Row $row){
        
        $row = $row->getRow();
        $this->prospectService->callForUpdatePartnerType($row);
        
    }
    
    public function insertPartnershipOfTypeClient(Row $row){
        
        $row = $row->getRow();
        $this->prospectService->callForInsertPartnershipOfTypeClient($row);
        
    }
    
    public function insertClient(Row $row){
    
        $row = $row->getRow();
        $this->prospectService->callForInsertClient($row);
        
    }
    
    public function deactivateProspect(Row $row){
        
        $row = $row->getRow();
        $this->prospectService->callForDeactivateProspect($row);
        
    }
    
    public function insertClientAddress(Row $row){
        
        $row = $row->getRow();
        $this->prospectService->callForInsertClientAddress($row);
        
    }
    
    public function selectProspectAddress(Row $row, Response $response){
        
        $row = $row->getRow();
        $returnedAddress = $this->prospectService->callForSelectAddressProspect($row);
        $response->addDataSet(new DataSet("DADOS", array("ADDRESS" => $returnedAddress)));
        
    }
    
    public function convertInClient(Row $row, DTO\Response $response){
        try {
            $row = $row->getRow();
            $nrparcnegocio = $this->parceiroService->replicaParceiro($row);
            if ($row['__is_new']) {
                $response->addDataset(new DataSet('columnsToReturn', array("NRPARCNEGOCIO" => $nrparcnegocio)));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }    
    }
    
    public function replicaInterlocutorProspect(Row $row, DTO\Response $response){
        $row = $row->getRow();
   
        if (!$row['__is_new']) {
            $this->prospectService->replicaInterlocutorProspect($row);
        }
    }
    
    public function setIdQualificadoProspect(Row $row){
        $row = $row->getRow();
        $this->prospectService->setIdQualificadoProspect($row);
    }
    
    public function findEvol(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'evolcont_parcnegocio';
        $this->parameterBagColumns = array('P_NRPARCNEGOCIO');
        parent::find($request, $response);
    }
    
    public function validaLead(DTO\Request $row, DTO\Response $response){
        try{
            $row = $row->getRow();
            
            //Define se a verificação será feita na tabela de lead ou de prospect(criação ou conversão de lead)
            if($row['LEAD'] === true){
                $row['FIELDNOME'] = 'NMCANDIDATO';
                $row['TABELA']    = 'CRM_CANDIDATO';
                $row['FIELDINSC'] = 'NRINSJURCAN';
            }else 
                if($row['PROSPECT'] === true){
                    $row['FIELDNOME'] = 'NMPRINCIPALPARC';
                    $row['TABELA']    = 'PARCNEGOCIO';
                    $row['FIELDINSC'] = 'NRINSCRICAOPARC';  
                }
            $return = $this->prospectService->validaLead($row);
            $leads = array();
            if(empty($return)){
                $leads = false;
            } else 
                if(is_array($return)){
                    foreach($return as $lead)
                        if(!in_array($lead, $leads)){
                            array_push($leads, $lead);
                            }
                } else {
                    $leads = true;
                }
            $response->addDataSet(new DataSet("DADOS", array("LEADS" => $leads)));
                
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public static function consultaRF(Row $row, Response $response) {
        $row = $row->getRow();
        $nrinsjurcan = $row['NUMBERINSC'];
        $consulta = ConsultaInscricaoRF::consultaDadoRF($nrinsjurcan);
        $response->addDataSet(new DataSet("CONSULTA", $consulta));
        
    }
    
    public function confirmationLocation(Row $row, Response $response){
        
        $row = $row->getRow();
        $resultadoConsulta = $row['RESULTCONSUL'];
        $resultadoConfirmado = self::confirState($resultadoConsulta);
        $resultadoConfirmado = array_merge($resultadoConfirmado , self::confirmCity($resultadoConsulta));
        $resultadoConfirmado = array_merge($resultadoConfirmado , self::confirmNeighborhood($resultadoConsulta, $resultadoConfirmado[1]['CDMUNICIPIO']));
        
        $response->addDataSet(new DataSet("LOCALIZACAO", $resultadoConfirmado));
    }
    
    public function confirState($consultaRF){
        $estadoConsultado = $consultaRF['Enderecos']['Estado'];
        $queryEstado = "SELECT ESTADO.SGESTADO AS SGESTADO,
                             ESTADO.NMESTADO AS NMESTADO
                        FROM ESTADO
                       WHERE UPPER(ESTADO.SGESTADO) = UPPER(:ESTCONSULT)
                         AND ESTADO.CDPAIS = '0055'";   
                         
        $params = array (
            "ESTCONSULT" => $estadoConsultado
        );
        
        try {
            $std = $this->connection->prepare($queryEstado);
            $std->execute($params);
            $retornoEstado = $std->fetchAll();
            
        } catch (\Exception $e){
            throw $e;
        }
        
        if($retornoEstado){
           return  $retornoEstado;
        } else {
            return array( 0 => array(
                "SGESTADO" => "", 
                "NMESTADO" => ""
            ));
        }
        
    }
    
    public function confirmCity($consultaRF){
        $sgestado   = $consultaRF['Enderecos']['Estado'];
        $codigoIbge = $consultaRF['Enderecos']['CodigoIBGE'];
        $queryMunicipio = "SELECT MUNICIPIO.CDMUNICIPIO AS CDMUNICIPIO,
                                 MUNICIPIO.NMMUNICIPIO AS NMMUNICIPIO
                            FROM MUNICIPIO
                           WHERE UPPER(MUNICIPIO.NMMUNICIPIO) = UPPER('".str_replace("'","''",$consultaRF['Enderecos']['Cidade'])."')
                             AND (CDMUNICIBGE = '$codigoIbge' OR '$codigoIbge' IS NULL)
                             AND MUNICIPIO.SGESTADO = '$sgestado'
                             AND MUNICIPIO.CDPAIS = '0055'
                             AND ROWNUM <= 1";
        
        try {
            $std = $this->connection->prepare($queryMunicipio);
            $std->execute();
            $retornoMunicipio = $std->fetchAll();
        } catch (\Exception $e){
            throw $e;
        }
        
        if($retornoMunicipio){
           return  $retornoMunicipio;
        } else {
            return array(0 => array(
                
                "CDMUNICIPIO" => "", 
                "NMMUNICIPIO" => ""
                
            ));
        }
    }
    
    public function confirmNeighborhood($consultaRF, $cdmunicipio){
        
        $sgestado = $consultaRF['Enderecos']['Estado'];
        $queryBairro = "SELECT BAIRRO.CDBAIRRO AS CDBAIRRO,
                               BAIRRO.NMBAIRRO   AS NMBAIRRO
                          FROM BAIRRO
                         WHERE UPPER(BAIRRO.NMBAIRRO) = UPPER('".str_replace("'","''",$consultaRF['Enderecos']['Bairro'])."')
                           AND BAIRRO.CDMUNICIPIO = '$cdmunicipio'
                           AND BAIRRO.SGESTADO    = '$sgestado'
                           AND BAIRRO.CDPAIS      = '0055'
                           AND ROWNUM <= 1";
        
        try {
            $std = $this->connection->prepare($queryBairro);
            $std->execute();
            $retornoBairro = $std->fetchAll();
        } catch (\Exception $e){
            throw $e;
        }
        
        if($retornoBairro && count($retornoBairro) != 0){
           return  $retornoBairro;
        } else 
            if($retornoBairro == array(0) || !$retornoBairro){
                return array(0 => array(
                    "CDBAIRRO" => "", 
                    "NMBAIRRO" => ""
                    
                ));
            }
    }
    
    public function insereEnderecoProspect(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $insere =  $row['INSERE'];
        
        if ($insere == true){
            $nrenderecoparc = $this->newCode->getCodeByTableName("ENDERECOPARC", $row["NRORG"]);
        } else {
            $nrenderecoparc = $row['NRENDERECOPARC'];
        }
        
        $nrparcnegocio    = $row['NRPARCNEGOCIO'];
        $cdtipoendereco   = $row['CDTIPOENDERECO'];
        $nrorg            = $row['NRORG'];
        $idativo          = $row['IDATIVO'];
        $dtinclusao       = $row['DTINCLUSAO'];
        $nrorginclusao    = $row['NRORGINCLUSAO'];
        $cdoperinclusao   = $this->environment->getCdOperador();
        $cdpais           = $row['CDPAIS'];
        $sgestado         = $row['SGESTADO'];
        $cdmunicipio      = $row['CDMUNICIPIO'];
        $cdbairro         = $row['CDBAIRRO'];
        $nmbairroendereco = $row['NMBAIRROENDERECO']; 
        $dscompleendereco = $row['DSCOMPLEENDERECO'];
        $nrcependereco    = $row['NRCEPENDERECO']; 
        $dsendereco       = $row['DSENDERECO'];
        $nrimovelendereco = $row['NRIMOVELENDERECO'];
        $dsreferenciaende = $row['DSREFERENCIAENDE'];
        $operador         = $this->environment->getCdOperador();

        $InsereDadosContr = $this->prospectService->insereEnderecoProspect($insere, $nrenderecoparc, $nrparcnegocio, $cdtipoendereco, $nrorg, $idativo, $dtinclusao, $nrorginclusao, $cdoperinclusao, $cdpais, $sgestado, $cdmunicipio, $cdbairro, $nmbairroendereco, $dscompleendereco, $nrcependereco, $dsendereco, $nrimovelendereco, $dsreferenciaende, $operador);
    } 
    
    public function insereInterlocutorProspect(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $insere =  $row['INSERE'];
        
        if ($insere == true){
            $nrseqinterlocutor = $this->newCode->getCodeByTableName("CRM_INTERLOCUTOR", $row["NRORG"]);
        } else {
             $nrseqinterlocutor = $row["NRSEQINTERLOCUTOR"];
        }
        
        $nminterlocutor     = $row["NMINTERLOCUTOR"];
        $nrparcnegocio      = $row["NRPARCNEGOCIO"];    
        $emailinterloc      = $row["EMAILINTERLOC"];                     
        $nrtelefone1        = $row["NRTELEFONE1"];                       
        $nrtelefone2        = $row["NRTELEFONE2"];                       
        $nrtelefone3        = $row["NRTELEFONE3"];                       
        $nrtelefone4        = $row["NRTELEFONE4"];                       
        $cdcargointerloc    = $row["CDCARGOINTERLOC"];                    
        $dtnascimento       = $row["DTNASCIMENTO"];                              
        $idrecebepresente   = $row["IDRECEBEPRESENTE"];                  
        $cdpais             = $row["CDPAIS"];                            
        $sgestado           = $row["SGESTADO"];                          
        $cdmunicipio        = $row["CDMUNICIPIO"];                       
        $cdbairro           = $row["CDBAIRRO"];                          
        $nrcepinterlocutor  = $row["NRCEPINTERLOCUTOR"];                 
        $dsendeinterlocutor = $row["DSENDEINTERLOCUTOR"];                
        $dscompinterlocutor = $row["DSCOMPINTERLOCUTOR"];                
        $nrendeinterlocutor = $row["NRENDEINTERLOCUTOR"];                      
        $nrorg              = $row["NRORG"];                
        $idativo            = $row["IDATIVO"];                  
        $dtinclusao         = $row["DTINCLUSAO"];                     
        $cdoperinclusao     = $this->environment->getCdOperador();                    
        $cdsetor            = $row["CDSETOR"];                            
        $nrcpfinterloc      = $row["NRCPFINTERLOC"];                     
        $operador           = $this->environment->getCdOperador();

        $InsereDadosContr = $this->prospectService->insereInterlocutorProspect($insere, $nrseqinterlocutor,$nminterlocutor, $nrparcnegocio, $emailinterloc, $nrtelefone1, $nrtelefone2, $nrtelefone3, $nrtelefone4, $cdcargointerloc, $dtnascimento, $idrecebepresente, $cdpais, $sgestado, $cdmunicipio, $cdbairro, $nrcepinterlocutor, $dsendeinterlocutor, $dscompinterlocutor, $nrendeinterlocutor, $nrorg, $idativo, $dtinclusao, $cdoperinclusao, $cdsetor, $nrcpfinterloc, $operador);
    }
    
    /**
     * Salva o novo operador responsável dos prospect selecionados
    **/
    public function salvarAlteracoesOpersResp(Request\Row $request, Response $response){
        try {
            $rows       = $request->getRow();
            $prospect   = $rows["CDPROSPECT"];
            $operador   = $rows["NRPARCRESPROSP"];
            $alteraProp = $rows["ALTERAPROPOSTA"];
            $alteraCont = $rows["ALTERACONTRATO"];
            $nrorg      = $this->environment->getNrorg();
            
            $this->prospectService->salvarAlteracoesOpersResp($prospect, $operador, $nrorg, $alteraProp, $alteraCont);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     /**
     * A função "buscaFilialHierarquia" busca a filial vinculado ao operador 
     * na hierarquiia de operadores (CRM_HIERARQUIAOPER)
     */
    public function buscaFilialHierarquia(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            
            $operador = $row["OPERADOR"];
            $nrorg    = $this->environment->getNrOrg();
            
            $filial = $this->prospectService->buscaFilialHierarquia($operador, $nrorg);
            $response->addDataSet(new DataSet("DATA", $filial));
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}

